#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@corp.ovh.com>


"""
import time
import logging
from cdumay_rest_client.client import RESTClient
from kser.controller import Controller

logger = logging.getLogger(__name__)


class LoopInfo(object):
    def __init__(self, sleep):
        self.sleep = sleep


class Consumer(object):
    REGISTRY = Controller

    def __init__(self, config, topics):
        self.client = RESTClient(
            server=config["bootstrap.servers"],
            username=config["sasl.username"], password=config["sasl.password"],
            timeout=config.get("timeout", 10)
        )
        self.topics = topics

    def call_kafka(self, topic, batch_size=1):
        logger.debug("{}.Checking {}".format(self.__class__.__name__, topic))
        data = self.client.do_request(
            method="GET", params=dict(limit=batch_size),
            path="/topic/{}".format(topic)
        )
        for item in data:
            str_msg = item.get("Value", None)
            if str_msg:
                self.REGISTRY.run(str_msg)

    def run(self, loopinfo=None, batch_size=1):
        """ Run consumer
        """
        logger.info("{}.Starting...".format(self.__class__.__name__))
        if loopinfo:
            while True:
                for topic in self.topics:
                    self.call_kafka(topic, batch_size)
                time.sleep(loopinfo.sleep)
        else:
            for topic in self.topics:
                self.call_kafka(topic, batch_size)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import logging
from uuid import uuid4

from kser import BaseSerializer
from cdumay_result import Result
from kser.transport import Message

logger = logging.getLogger(__name__)


class EntrypointMeta(type):
    @property
    def path(cls):
        return "{}.{}".format(cls.__module__, cls.__name__)


class Entrypoint(BaseSerializer, metaclass=EntrypointMeta):
    @classmethod
    def _prerun(cls, kmsg):
        """ To execute before running message

        :param kser.transport.Message kmsg: Kafka message
        :return: Kafka message
        :rtype: kser.transport.Message
        """
        logger.debug("{}.PreRun: {}[{}]".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid
        ))
        return cls.prerun(kmsg)

    @classmethod
    def prerun(cls, kmsg):
        """ To implement

        :param kser.transport.Message kmsg: Kafka message
        :return: Kafka message
        :rtype: kser.transport.Message
        """
        return kmsg

    @classmethod
    def _postrun(cls, kmsg, result):
        """ To execute after exection

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.debug("{}.PostRun: {}[{}]".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid
        ))
        return cls.postrun(kmsg, result)

    @classmethod
    def postrun(cls, kmsg, result):
        """ To implement

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        return result

    @classmethod
    def _run(cls, kmsg):
        """ Execution body

        :param kser.transport.Message kmsg: Kafka message
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.debug("{}.Run: {}[{}]".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid
        ))
        return cls.run(uuid=kmsg.uuid, result=kmsg.result, **kmsg.params)

    @classmethod
    def run(cls, uuid, result, **kwargs):
        """ To implement

        :param str uuid: Message UUID
        :param kser.result.Result result: Previous result
        :param dict kwargs: Parameters
        :return: Execution result
        :rtype: kser.result.Result
        """
        return Result(uuid=uuid, retval=kwargs)

    @classmethod
    def execute(cls, kmsg):
        """ Execution 'wrapper' to make sure that it return a result

        :param kser.transport.Message kmsg: Kafka message
        :return: Execution result
        :rtype: kser.result.Result
        """
        try:
            result = cls._onsuccess(
                kmsg, cls._postrun(kmsg, cls._run(cls._prerun(kmsg)))
            )

        except Exception as exc:
            result = cls._onerror(
                kmsg, Result.fromException(exc, uuid=kmsg.uuid)
            )

        finally:
            return result

    @classmethod
    def as_kmsg(cls, uuid=None, params=None, result=None):
        if not uuid:
            uuid = str(uuid4())
        if not params:
            params = dict()

        return Message(
            uuid=uuid, entrypoint=cls.path, params=params, result=result
        )

    @classmethod
    def self_execute(cls, uuid=None, params=None, result=None):
        return cls.execute(cls.as_kmsg(uuid=uuid, params=params, result=result))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import logging
import socket

__hostname__ = socket.gethostname()

logger = logging.getLogger(__name__)


class BaseSerializer(object):
    @classmethod
    def _onsuccess(cls, kmsg, result):
        """ To execute on execution success

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.info("{}.Success: {}[{}]: {}".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid, result
        ))
        return cls.onsuccess(kmsg, result)

    @classmethod
    def onsuccess(cls, kmsg, result):
        """ To execute on execution success

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        return result

    @classmethod
    def _onerror(cls, kmsg, result):
        """ To execute on execution failure

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.error("{}.Failed: {}[{}]: {}".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid, result
        ), extra=result.retval)
        return cls.onerror(kmsg, result)

    @classmethod
    def onerror(cls, kmsg, result):
        """ To implement

        :param kser.transport.Message kmsg: Kafka message
        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        return result


class BaseTransportSerializer(BaseSerializer):
    @classmethod
    def _onmessage(cls, kmsg):
        """ Call on received message

        :param kser.transport.Message kmsg: Kafka message
        :return: Kafka message
        :rtype: kser.transport.Message
        """
        logger.debug("{}.ReceivedMessage: {}[{}]".format(
            cls.__name__, kmsg.entrypoint, kmsg.uuid
        ))
        return cls.onmessage(kmsg)

    @classmethod
    def onmessage(cls, kmsg):
        """ To implement

        :param kser.transport.Message kmsg: Kafka message
        :return: Kafka message
        :rtype: kser.transport.Message
        """
        return kmsg



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
from marshmallow import Schema, fields
from cdumay_rest_client.exceptions import ValidationError
from cdumay_result import ResultSchema, Result


class BaseSchema(Schema):
    uuid = fields.String(required=True)
    entrypoint = fields.String(required=True)
    params = fields.Dict(default=dict)


class Base(object):
    MARSHMALLOW_SCHEMA = BaseSchema()

    def __init__(self, uuid, entrypoint, params=None):
        self.uuid = uuid
        self.entrypoint = entrypoint
        self.params = params if params else dict()

    def dump(self):
        """description of dump"""
        return self.MARSHMALLOW_SCHEMA.dump(self).data

    def dumps(self):
        """description of dumps"""
        return self.MARSHMALLOW_SCHEMA.dumps(self).data

    def __str__(self):
        return str(self.dump())


class RouteSchema(BaseSchema):
    onerror = fields.String()


class Route(Base):
    MARSHMALLOW_SCHEMA = RouteSchema()

    def __init__(self, uuid, entrypoint, params=None, onerror=None):
        Base.__init__(self, uuid, entrypoint, params)
        self.onerror = onerror or entrypoint

    def __repr__(self):
        """"""
        return "Route<uuid='{}', entrypoint='{}', onerror={}>".format(
            self.uuid, self.entrypoint, self.onerror
        )


class MessageSchema(BaseSchema):
    result = fields.Nested(ResultSchema, missing=None)
    route = fields.Nested(RouteSchema, missing=None)


class Message(Base):
    MARSHMALLOW_SCHEMA = MessageSchema()

    @classmethod
    def loads(cls, json_data):
        """description of load"""
        data, errors = cls.MARSHMALLOW_SCHEMA.loads(json_data)
        if len(errors) > 0:
            raise ValidationError(
                "Invalid Message", extra=dict(**data, **errors)
            )
        return cls(**data)

    def __init__(self, uuid, entrypoint, params=None, result=None, route=None):
        Base.__init__(self, uuid, entrypoint, params)
        if result:
            if isinstance(result, Result):
                self.result = result
            else:
                self.result = Result(**result)
        else:
            self.result = Result(uuid=uuid)

        if route:
            if isinstance(route, Route):
                self.route = route
            else:
                self.route = Route(**route)
        else:
            self.route = None

    def __repr__(self):
        """"""
        return "Message<uuid='{}', entrypoint='{}', result={}>".format(
            self.uuid, self.entrypoint, self.result
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@corp.ovh.com>


"""
from kser import BaseTransportSerializer
from cdumay_result import Result
from cdumay_rest_client.client import RESTClient


class Producer(BaseTransportSerializer):
    def __init__(self, config):
        """ Create new Producer instance using provided configuration dict.

        :param dict config: configuration
        """
        self.client = RESTClient(
            server=config["bootstrap.servers"],
            username=config["sasl.username"], password=config["sasl.password"],
            timeout=config.get("timeout", 10)
        )

    def bulk_send(self, topic, kmsgs):
        """ Send a batch of messages

        :param str topic: a kafka topic
        :param ksr.transport.Message kmsgs: Messages to serialize
        :return: Execution result
        :rtype: kser.result.Result
        """

        try:
            self.client.do_request(
                method="POST", path="/topic/{}".format(topic), data=[
                    dict(Value=k.MARSHMALLOW_SCHEMA.dump(k).data) for k in kmsgs
                ]
            )
            return Result(stdout="{} message(s) sent".format(len(kmsgs)))

        except Exception as exc:
            return Result.fromException(exc)

    def send(self, topic, kmsg):
        """ Send the message into the given topic

        :param str topic: a kafka topic
        :param ksr.transport.Message kmsg: Message to serialize
        :return: Execution result
        :rtype: kser.result.Result
        """
        try:
            self.client.do_request(
                method="POST", params=dict(format="raw"),
                path="/topic/{}".format(topic),
                data=kmsg.MARSHMALLOW_SCHEMA.dump(kmsg).data
            )
            result = Result(
                uuid=kmsg.uuid, stdout="Message sent: {} ({})".format(
                    kmsg.uuid, kmsg.entrypoint
                )
            )

        except Exception as exc:
            result = Result.fromException(exc, kmsg.uuid)

        finally:
            if result.retcode < 300:
                return self._onsuccess(kmsg=kmsg, result=result)
            else:
                return self._onerror(kmsg=kmsg, result=result)
